@@ -7,8 +7,11 @@ pushd "$(dirname "$0")"
 [ ! $EMSDK ] && pushd emsdk && source ./emsdk_env.sh && popd
 
 # Compile LLVM bitcode
+# XXX FIXME: We'd like to enable pcre2, but somehow,
+# this fails in the github automated testing environment.
 pushd ../..
-emconfigure ./configure --disable-editline --disable-sat-solver --disable-java-bindings --disable-python-bindings
+emconfigure ./configure --disable-editline --disable-sat-solver --disable-java-bindings --disable-python-bindings --disable-pcre2
+emmake make clean
 emmake make
 popd
 
