@@ -1028,7 +1028,7 @@ class XLookupListTestCase(unittest.TestCase):
         for i in range(len(dictnode)):
             self.assertIn("(", str(dictnode[i].exp), "Missing expression")
             if dictnode[i].string == sm('test.n'):
-                self.assertEqual(dictnode[i].file, 'en/words/words.n.1-const')
+                self.assertEqual(dictnode[i].file, 'en/words/words.n.4-const')
             elif dictnode[i].string == sm('test.v'):
                 self.assertIsNone(dictnode[i].file)
 
@@ -1093,7 +1093,7 @@ class XExp_resolving_test(unittest.TestCase):
         Test expression resolving using the default headline:4 setting from
         data/en/4.0.dialect.
         """
-        dictnode = clg.dictionary_lookup_list(self.d._obj, sm('test.n'))
+        dictnode = clg.dictionary_lookup_list(self.d._obj, sm('book.n'))
         exp_old = dictnode[0].exp
         exp_new = clg.lg_exp_resolve(self.d._obj, exp_old, ParseOptions()._obj)  # headline:4
 
@@ -1101,13 +1101,14 @@ class XExp_resolving_test(unittest.TestCase):
         # exp_old to exp_new and validate them.
         # Locations 2 and 4 are in exp_old. Locations 6 and 7 are in exp_new.
         # () is the null expression that is controlled by "headline".
+        # 99.000 is the current headline cost placed into 4.0.dialect
         str_comb = str(exp_old) + '%' + str(exp_new)
         diff = re.search(r'^([^%]*)(\(Ds\*\*x- or \([^%]*?\)\)\)\))([^%]*)(\(Ds\*\*c- or \([^%]*?\)\)\)\))([^%]*)%\1(.*)\3(.*)\5$', str_comb)
         #                      1                2                     3                4                     5        6     7
         self.assertEqual(diff.group(2), '(Ds**x- or (())))')
         self.assertEqual(diff.group(4), '(Ds**c- or (())))')
-        self.assertEqual(diff.group(6), '(Ds**x- or ([[[[()]]]])))')
-        self.assertEqual(diff.group(7), '(Ds**c- or ([[[[()]]]])))')
+        self.assertEqual(diff.group(6), '(Ds**x- or ([()]99.000)))')
+        self.assertEqual(diff.group(7), '(Ds**c- or ([()]99.000)))')
 
 
 # Currently, the dictionary creating function sets the generation mode if
@@ -1340,7 +1341,7 @@ class ZZdict_display_word_expr(unittest.TestCase):
        self.assertTrue(len(list(out.splitlines())) > 1800)
 
     def test_macros(self):
-       out = clg.dict_display_word_expr(self.d._obj, 'test/m', self.po._obj)
+       out = clg.dict_display_word_expr(self.d._obj, 'book/m', self.po._obj)
        self.assertIn('<common-const-noun>:', out)
        self.assertIn('<verb-pl,i>', out)
 
