@@ -25,9 +25,10 @@
 #include "post-process/post-process.h"  // post_process_new
 #include "prepare/exprune.h"
 #include "string-set.h"
-#include "tokenize//wordgraph.h"        // wordgraph_delete
+#include "tokenize/wordgraph.h"         // wordgraph_delete
 #include "resources.h"
 #include "sat-solver/sat-encoder.h"
+#include "tokenize/lookup-exprs.h"
 #include "tokenize/tokenize.h"
 #include "tokenize/word-structures.h"   // Word_struct
 #include "utilities.h"
@@ -35,7 +36,7 @@
 /* Its OK if this is racey across threads.  Any mild shuffling is enough. */
 static unsigned int global_rand_state = 0;
 
-int verbosity;
+int verbosity = D_USER_BASIC;
 /* debug and test should not be NULL since they can be used before they
  * are assigned a value by parse_options_get_...() */
 char * debug = (char *)"";
@@ -47,28 +48,6 @@ char * test = (char *)"";
 *
 ****************************************************************/
 
-/**
- * For sorting the linkages in postprocessing
- */
-
-static int VDAL_compare_parse(Linkage l1, Linkage l2)
-{
-	Linkage_info * p1 = &l1->lifo;
-	Linkage_info * p2 = &l2->lifo;
-
-	if (p1->N_violations != p2->N_violations) {
-		return (p1->N_violations - p2->N_violations);
-	}
-	else if (p1->unused_word_cost != p2->unused_word_cost) {
-		return (p1->unused_word_cost - p2->unused_word_cost);
-	}
-	else if (p1->disjunct_cost > p2->disjunct_cost) return 1;
-	else if (p1->disjunct_cost < p2->disjunct_cost) return -1;
-	else {
-		return (p1->link_cost - p2->link_cost);
-	}
-}
-
 /**
  * Create and initialize a Parse_Options object
  */
@@ -81,16 +60,15 @@ Parse_Options parse_options_create(void)
 
 	/* Here's where the values are initialized */
 
-	/* The parse_options_set_(verbosity|debug|test) functions set also the
-	 * corresponding global variables. So these globals are initialized
-	 * here too. */
-	verbosity = po->verbosity = 1;
+	/* The parse_options_set_(verbosity|debug|test) functions also
+	 * set the corresponding global variables. Set them here, also.
+	 * Note that this is usally called *after* dictionary creation! */
+	verbosity = po->verbosity = D_USER_BASIC;
 	debug = po->debug = (char *)"";
 	test = po->test = (char *)"";
 
-	/* A cost of 2.7 allows the usual cost-2 connectors, plus the
-	 * assorted fractional costs, without going to cost 3.0, which
-	 * is used only during panic-parsing. */
+	/* Set dijunct_cost to a bogus value of -10000. The dict-common
+	 * code will set this to a more reasonable default. */
 	po->disjunct_cost = UNINITIALIZED_MAX_DISJUNCT_COST;
 	po->min_null_count = 0;
 	po->max_null_count = 0;
@@ -99,13 +77,12 @@ Parse_Options parse_options_create(void)
 	po->use_sat_solver = false;
 #endif
 	po->linkage_limit = 100;
-#if defined HAVE_HUNSPELL || defined HAVE_ASPELL
-	po->use_spell_guess = 7;
-#else
-	po->use_spell_guess = 0;
-#endif /* defined HAVE_HUNSPELL || defined HAVE_ASPELL */
 
-	po->cost_model.compare_fn = &VDAL_compare_parse;
+	// Disable spell-guessing by default. Aspell 0.60.8 and possibly
+	// others leak memory.
+	po->use_spell_guess = 0;
+
+	po->cost_model.compare_fn = &VDAL_compare_linkages;
 	po->cost_model.type = VDAL;
 	po->short_length = 16;
 	po->all_short = false;
@@ -133,7 +110,7 @@ void parse_options_set_cost_model_type(Parse_Options opts, Cost_Model_type cm)
 	switch(cm) {
 	case VDAL:
 		opts->cost_model.type = VDAL;
-		opts->cost_model.compare_fn = &VDAL_compare_parse;
+		opts->cost_model.compare_fn = &VDAL_compare_linkages;
 		break;
 	default:
 		prt_error("Error: Illegal cost model: %d\n", (int)cm);
@@ -478,9 +455,6 @@ Sentence sentence_create(const char *input_string, Dictionary dict)
 
 int sentence_split(Sentence sent, Parse_Options opts)
 {
-	Dictionary dict = sent->dict;
-	bool fw_failed = false;
-
 	/* 0 == global_rand_state denotes "repeatable rand".
 	 * If non-zero, set it here so that anysplit can use it.
 	 */
@@ -496,30 +470,24 @@ int sentence_split(Sentence sent, Parse_Options opts)
 		return -1;
 	}
 
+	Dictionary dict = sent->dict;
 	if (!setup_dialect(dict, opts))
 		return -4;
 
-	/* Flatten the word graph created by separate_sentence() to a 2D-word-array
-	 * which is compatible to the current parsers.
-	 * This may fail if UNKNOWN_WORD is needed but
-	 * is not defined in the dictionary, or an internal error happens. */
-	fw_failed = !flatten_wordgraph(sent, opts);
+	/* Flatten the word graph created by separate_sentence() to a
+	 * 2D-word-array which is compatible to the current parsers. */
+	flatten_wordgraph(sent, opts);
 
-	/* If unknown_word is not defined, then no special processing
-	 * will be done for e.g. capitalized words. */
-	if (!(dict->unknown_word_defined && dict->use_unknown_word))
+	/* This may fail if UNKNOWN_WORD is needed, but is not defined in
+	 * the dictionary. In that case, warnings for the unknown word were
+	 * already printed. */
+	if (!build_sentence_expressions(sent, opts))
 	{
-		if (!sentence_in_dictionary(sent)) {
-			return -2;
-		}
-	}
+		err_ctxt ec = { sent };
+		err_msgc(&ec, lg_Error,
+			"Cannot parse sentence with unknown words!\n");
 
-	if (fw_failed)
-	{
-		/* Make sure an error message is always printed.
-		 * So it may be redundant. */
-		prt_error("Error: sentence_split(): Internal error detected\n");
-		return -3;
+		return -2;
 	}
 
 	if (verbosity >= D_USER_TIMES)
@@ -527,22 +495,12 @@ int sentence_split(Sentence sent, Parse_Options opts)
 	return 0;
 }
 
-static void free_sentence_words(Sentence sent)
-{
-	for (WordIdx i = 0; i < sent->length; i++)
-	{
-		free(sent->word[i].alternatives);
-	}
-	free_sentence_disjuncts(sent, /*categories_too*/true);
-	free((void *) sent->word);
-	sent->word = NULL;
-}
-
 void sentence_delete(Sentence sent)
 {
 	if (!sent) return;
 	sat_sentence_delete(sent);
-	free_sentence_words(sent);
+	free_sentence_disjuncts(sent, /*categories_too*/true);
+	free_words(sent);
 	wordgraph_delete(sent);
 	string_set_delete(sent->string_set);
 	free_linkages(sent);
@@ -553,7 +511,7 @@ void sentence_delete(Sentence sent)
 
 	global_rand_state = sent->rand_state;
 	pool_delete(sent->Match_node_pool);
-	pool_delete(sent->Table_connector_pool);
+	pool_delete(sent->Table_tracon_pool);
 	pool_delete(sent->wordvec_pool);
 	pool_delete(sent->Exp_pool);
 	pool_delete(sent->X_node_pool);
@@ -647,9 +605,8 @@ int sentence_link_cost(Sentence sent, LinkageIdx i)
 
 int sentence_parse(Sentence sent, Parse_Options opts)
 {
-	int rc;
-
-	if (IS_GENERATION(sent->dict))
+	Dictionary dict = sent->dict;
+	if (IS_GENERATION(dict))
 	{
 #if USE_SAT_SOLVER
 		if (opts->use_sat_solver)
@@ -666,7 +623,7 @@ int sentence_parse(Sentence sent, Parse_Options opts)
 	}
 
 	if (opts->disjunct_cost == UNINITIALIZED_MAX_DISJUNCT_COST)
-		opts->disjunct_cost = sent->dict->default_max_disjunct_cost;
+		opts->disjunct_cost = dict->default_max_disjunct_cost;
 
 	sent->num_valid_linkages = 0;
 
@@ -676,7 +633,7 @@ int sentence_parse(Sentence sent, Parse_Options opts)
 	 */
 	if (0 == sent->length)
 	{
-		rc = sentence_split(sent, opts);
+		int rc = sentence_split(sent, opts);
 		if (rc) return -1;
 	}
 	else
@@ -697,12 +654,6 @@ int sentence_parse(Sentence sent, Parse_Options opts)
 	}
 
 	resources_reset(opts->resources);
-
-	/* When a dynamic dictionary is used, expressions are read on demand,
-	 * so the connector descriptor table is not yet ready at this point. */
-	if (IS_DYNAMIC_DICT(sent->dict))
-		condesc_setup(sent->dict);
-
 	for (WordIdx w = 0; w < sent->length; w++)
 	{
 		for (X_node *x = sent->word[w].x; x != NULL; x = x->next)
@@ -714,6 +665,7 @@ int sentence_parse(Sentence sent, Parse_Options opts)
 	 */
 	expression_prune(sent, opts);
 	print_time(opts, "Finished expression pruning");
+
 #if USE_SAT_SOLVER
 	if (opts->use_sat_solver)
 	{
