@@ -283,13 +283,13 @@ static void connector_encode_lc(const char *lc_string, condesc_t *desc)
 }
 
 /**
- * Calculate fixed connector information that only depend on its string.
+ * Calculate fixed connector information that depends only on its string.
  * This information is used to speed up the parsing stage. It is
- * calculated during the directory creation and doesn't change afterward.
+ * calculated during the dictionary creation, and doesn't change afterward.
  *
  * Note: check_connector() has already validated the connector string.
  */
-static void calculate_connector_info(condesc_t * c)
+void calculate_connector_info(condesc_t * c)
 {
 	const char *s;
 
@@ -365,7 +365,7 @@ static uint32_t connector_str_hash(const char *s)
 /**
  * Compare connector UC parts, for qsort.
  */
-static int condesc_by_uc_constring(const void * a, const void * b)
+int condesc_by_uc_constring(const void * a, const void * b)
 {
 	const condesc_t * const * cda = a;
 	const condesc_t * const * cdb = b;
@@ -406,7 +406,7 @@ static int condesc_by_uc_constring(const void * a, const void * b)
  * Enumerate the connectors by their UC parts - equal parts get the same number.
  * It can later serve as a table index, as if it was a perfect hash.
  */
-bool sort_condesc_by_uc_constring(Dictionary dict)
+static bool sort_condesc_by_uc_constring(Dictionary dict)
 {
 	if ((0 == dict->contable.num_con) && !IS_DYNAMIC_DICT(dict))
 	{
@@ -550,6 +550,7 @@ condesc_t *condesc_add(ConTable *ct, const char *constring)
 		lgdebug(+11, "Creating connector '%s' (%zu)\n", constring, ct->num_con);
 		h->desc = pool_alloc(ct->mempool);
 		h->desc->string = constring;
+		h->desc->uc_num = UINT32_MAX;
 		h->str_hash = hash;
 		ct->num_con++;
 
@@ -567,11 +568,18 @@ void condesc_init(Dictionary dict, size_t num_con)
 {
 	ConTable *ct = &dict->contable;
 
-	condesc_table_alloc(ct, num_con);
 	ct->mempool = pool_new(__func__, "ConTable",
-								  /*num_elements*/1024, sizeof(condesc_t),
+								  /*num_elements*/num_con, sizeof(condesc_t),
 								  /*zero_out*/true, /*align*/true, /*exact*/false);
 
+	// Connector hash table must be an exact power of two.
+	int nbits = 0;
+	while (num_con) { nbits++; num_con >>= 1; }
+
+	// 4 times larger, to provide a reasonable hash table load factor.
+	nbits += 2;
+	condesc_table_alloc(ct, 1<<nbits);
+
 	ct->length_limit_def = NULL;
 	ct->length_limit_def_next = &ct->length_limit_def;
 }
@@ -582,4 +590,5 @@ void condesc_setup(Dictionary dict)
 	set_all_condesc_length_limit(dict);
 	free(dict->contable.sdesc);
 }
+
 /* ========================= END OF FILE ============================== */
