@@ -46,15 +46,16 @@ bool cost_eq(float cost1, float cost2)
  */
 const char *cost_stringify(float cost)
 {
-	static TLS char buf[16];
+	static TLS char buf[COST_MAX_DEC_PLACES+8];
+	static const float scale = EXP10F(1, COST_MAX_DEC_PLACES);
 
-	const int scale = (int)EXP10F(1, COST_MAX_DEC_PLACES);
-	const bool sign = signbit(cost);
-	const float roundinc = (1 / EXP10F(2, COST_MAX_DEC_PLACES));
-	int c = (int)((fabsf(cost) + roundinc) * scale);
+	bool sign = signbit(cost);
+	float ac = fabsf(cost);
+	int c = (int) floorf(ac);
+	unsigned long r = (unsigned long) roundf((ac-c)*scale);
 
-	int l = snprintf(buf, sizeof(buf), "%s%d.%0*d", sign ? "-" : "",
-	                 c / scale, COST_MAX_DEC_PLACES, c - c / scale * scale);
+	int l = snprintf(buf, sizeof(buf), "%s%d.%0*lu", sign ? "-" : "",
+	                 c, COST_MAX_DEC_PLACES, r);
 	if ((l < 0) || (l >= (int)sizeof(buf))) return "ERR_COST";
 
 	return buf;
@@ -361,7 +362,7 @@ char *lg_exp_stringify(const Exp *n)
  * cumbersome. Care should be taken due to the static memory of the result. */
 const char *exp_stringify(const Exp *n)
 {
-	static TLS char *s;
+	static TLS char *s = NULL;
 
 	free(s);
 	s = NULL;
@@ -460,7 +461,7 @@ static bool is_ASAN_uninitialized(uintptr_t a)
 	return (a == asan_uninitialized);
 }
 
-void prt_exp_all(dyn_str *s, Exp *e, int i, Dictionary dict)
+static void prt_exp_all(dyn_str *s, Exp *e, int i, Dictionary dict)
 {
 	if (is_ASAN_uninitialized((uintptr_t)e))
 	{
