@@ -13,11 +13,11 @@
 
 #include <string.h>
 
-#include "api-structures.h"             // Sentence_s (add_empty_word)
 #include "connectors.h"
 #include "dict-common/dialect.h"
 #include "dict-common/dict-affix.h"     // is_stem
 #include "dict-common/dict-common.h"
+#include "dict-common/dict-internals.h"
 #include "dict-common/dict-utils.h"     // patch_subscript
 #include "dict-common/file-utils.h"
 #include "dict-common/idiom.h"
@@ -332,9 +332,9 @@ static bool link_advance(Dictionary dict)
 		 * into the token. This includes embedded white-space.
 		 *
 		 * This is slightly awkward, and is NOT conventional
-		 * programming practice of quotes. It does allow quotes to be
+		 * programming practice for quotes. It does allow quotes to be
 		 * embedded into the middle of tokens. It misbehaves mildly
-		 * when a quotes string is used with a #define statement.
+		 * when a quoted string is used with a #define statement.
 		 */
 		if (quote_mode) {
 
@@ -520,7 +520,6 @@ static Exp * make_connector(Dictionary dict)
 		dn = strict_lookup_list(dict, dict->token);
 		if (dn == NULL)
 		{
-			dict_node_free_list(dn);
 			dict_error2(dict, "Perhaps missing + or - in a connector.\n"
 			                 "Or perhaps you forgot the subscript on a word.\n"
 			                 "Or perhaps the word is used before it is defined:",
@@ -584,42 +583,6 @@ static Exp * make_connector(Dictionary dict)
 	return n;
 }
 
-/* ======================================================================== */
-/* Empty-word handling. */
-
-/** Insert ZZZ+ connectors.
- *  This function was mainly used to support using empty-words, a concept
- *  that has been eliminated. However, it is still used to support linking of
- *  quotes that don't get the QUc/QUd links.
- */
-void add_empty_word(Sentence sent, X_node *x)
-{
-	Exp *zn, *an;
-	const char *ZZZ = string_set_lookup(EMPTY_CONNECTOR, sent->dict->string_set);
-	/* This function is called only if ZZZ is in the dictionary. */
-
-	/* The left-wall already has ZZZ-. The right-wall will not arrive here. */
-	if (MT_WALL == x->word->morpheme_type) return;
-
-	/* Replace plain-word-exp by {ZZZ+} & (plain-word-exp) in each X_node.  */
-	for(; NULL != x; x = x->next)
-	{
-		/* Ignore stems for now, decreases a little the overhead for
-		 * stem-suffix languages. */
-		if (is_stem(x->string)) continue; /* Avoid an unneeded overhead. */
-		//lgdebug(+0, "Processing '%s'\n", x->string);
-
-		/* zn points at {ZZZ+} */
-		zn = make_connector_node(sent->dict, sent->Exp_pool, ZZZ, '+', false);
-		zn = make_optional_node(sent->Exp_pool, zn);
-
-		/* an will be {ZZZ+} & (plain-word-exp) */
-		an = make_and_node(sent->Exp_pool, zn, x->exp);
-
-		x->exp = an;
-	}
-}
-
 /* ======================================================================== */
 
 /**
@@ -738,7 +701,7 @@ static Exp *make_expression(Dictionary dict)
 					           badchar);
 					return NULL;
 				}
-				if (nl->tag_type != Exptag_none)
+				if ((nl->type == CONNECTOR_type) || (nl->tag_type != Exptag_none))
 				{
 					nl = make_unary_node(dict->Exp_pool, nl);
 				}
@@ -1058,7 +1021,7 @@ static bool read_entry(Dictionary dict)
 			dict->already_got_it = save_already_got_it;
 			dict->line_number    = save_line_number;
 
-			free(instr);
+			free_file_contents(instr);
 			if (!rc) goto syntax_error;
 
 			/* when we return, point to the next entry */
