@@ -18,12 +18,8 @@ struct Word_file_struct
 {
 	Word_file * next;
 	const char *file;    /* the file name */
-	bool changed;        /* TRUE if this file has been changed (XXX unused) */
 };
 
 void free_Word_file(Word_file * wf);
 
 Dict_node * read_word_file(Dictionary dict, Dict_node * dn, char * filename);
-
-void patch_subscript(char *);
-
