@@ -173,6 +173,7 @@ void free_sentence_disjuncts(Sentence sent, bool category_too)
 		pool_delete(sent->Disjunct_pool);
 		pool_delete(sent->Connector_pool);
 		sent->Disjunct_pool = NULL;
+		sent->Connector_pool = NULL;
 	}
 }
 
@@ -336,84 +337,6 @@ static void disjunct_dup_table_delete(disjunct_dup_table *dt)
 	free(dt);
 }
 
-#ifdef DEBUG
-GNUC_UNUSED static int gword_set_len(const gword_set *gl)
-{
-	int len = 0;
-	for (; NULL != gl; gl = gl->next) len++;
-	return len;
-}
-#endif
-
-/**
- * Return a new gword_set element, initialized from the given element.
- * @param old_e Existing element.
- */
-static gword_set *gword_set_element_new(gword_set *old_e)
-{
-	gword_set *new_e = malloc(sizeof(gword_set));
-	*new_e = (gword_set){0};
-
-	new_e->o_gword = old_e->o_gword;
-	gword_set *chain_next = old_e->chain_next;
-	old_e->chain_next = new_e;
-	new_e->chain_next = chain_next;
-
-	return new_e;
-}
-
-/**
- * Add an element to existing gword_set. Uniqueness is assumed.
- * @return A new set with the element.
- */
-static gword_set *gword_set_add(gword_set *gset, gword_set *ge)
-{
-	gword_set *n = gword_set_element_new(ge);
-	n->next = gset;
-	gset = n;
-
-	return gset;
-}
-
-/**
- * Combine the given gword sets.
- * The gword sets are not modified.
- * This function is used for adding the gword pointers of an eliminated
- * disjunct to the ones of the kept disjuncts, with no duplicates.
- *
- * @param kept gword_set of the kept disjunct.
- * @param eliminated gword_set of the eliminated disjunct.
- * @return Use copy-on-write semantics - the gword_set of the kept disjunct
- * just gets returned if there is nothing to add to it. Else - a new gword
- * set is returned.
- */
-static gword_set *gword_set_union(gword_set *kept, gword_set *eliminated)
-{
-	/* Preserve the gword pointers of the eliminated disjunct if different. */
-	gword_set *preserved_set = NULL;
-	for (gword_set *e = eliminated; NULL != e; e = e->next)
-	{
-		gword_set *k;
-
-		/* Ensure uniqueness. */
-		for (k = kept; NULL != k; k = k->next)
-			if (e->o_gword == k->o_gword) break;
-		if (NULL != k) continue;
-
-		preserved_set = gword_set_add(preserved_set, e);
-	}
-
-	if (preserved_set)
-	{
-		/* Preserve the originating gword pointers of the remaining disjunct. */
-		for (gword_set *k = kept; NULL != k; k = k->next)
-			preserved_set = gword_set_add(preserved_set, k);
-		kept = preserved_set;
-	}
-
-	return kept;
-}
-
 /**
  * Takes the list of disjuncts pointed to by d, eliminates all
  * duplicates, and returns a pointer to a new list.
@@ -554,12 +477,15 @@ void count_disjuncts_and_connectors(Sentence sent, unsigned int *dca,
 
 	for (WordIdx w = 0; w < sent->length; w++)
 	{
+		unsigned int ndw = 0;
 		for (Disjunct *d = sent->word[w].d; d != NULL; d = d->next)
 		{
-			dcnt++;
+			ndw++;
 			for (Connector *c = d->left; c != NULL; c = c->next) ccnt++;
 			for (Connector *c = d->right; c !=NULL; c = c->next) ccnt++;
 		}
+		sent->word[w].num_disjuncts = ndw;
+		dcnt += ndw;
 	}
 
 	*cca = ccnt;
@@ -632,7 +558,7 @@ void count_disjuncts_and_connectors(Sentence sent, unsigned int *dca,
  * linkages per a given connector-pair using connector addresses. Since
  * an exhaustive search is done, such an approach has two main problems
  * for long sentences:
- * 1. A very big count hash table (Table_connector in count.c) is used
+ * 1. A very big count hash table (Table_tracon in count.c) is used
  * due to the huge number of connectors (100Ks) in long sentences, a
  * thing that causes a severe CPU cache trash (to the degree that
  * absolutely most of the memory accesses are L3 misses).
@@ -880,12 +806,6 @@ static Disjunct *pack_disjuncts(Sentence sent, Tracon_sharing *ts,
 
 #define TLSZ 8192         /* Initial size of the tracon list table */
 
-/* Reserved tracon ID space for NULL connectors (zero-length tracons).
- * Currently, tracons are unique per word. So this is actually the max.
- * number of words in a sentence rounded up to a power of 2.
- * FIXME: Derive it from MAX_SENTENCE. */
-#define WORD_OFFSET 256
-
 /** Create a context descriptor for disjuncts & connector memory "packing".
  *   Allocate a memory block for all the disjuncts & connectors.
  *   The current Connector struct size is 32 bytes, and the intention is
@@ -947,7 +867,7 @@ static Tracon_sharing *pack_sentence_init(Sentence sent, bool is_pruning)
 	ts->dblock = dblock;
 	ts->num_connectors = ccnt;
 	ts->num_disjuncts = dcnt;
-	ts->word_offset = is_pruning ? 1 : WORD_OFFSET;
+	ts->word_offset = is_pruning ? 1 : NULL_TRACON_BLOCK;
 	ts->is_pruning = is_pruning;
 	ts->next_id[0] = ts->next_id[1] = ts->word_offset;
 	ts->last_token = (uintptr_t)-1;
@@ -1031,6 +951,12 @@ void free_tracon_sharing(Tracon_sharing *ts)
 	free(ts);
 }
 
+void free_tracon_memblock(Tracon_sharing *ts)
+{
+	free(ts->memblock);
+	free_tracon_sharing(ts);
+}
+
 /**
  * Pack all disjunct and connectors into one big memory block, share
  * tracon memory and generate tracon IDs (for parsing) or tracon lists
@@ -1145,3 +1071,8 @@ void restore_disjuncts(Sentence sent, void *saved_memblock, Tracon_sharing *ts)
 
 	memcpy(ts->memblock, saved_memblock, ts->memblock_sz);
 }
+
+void free_saved_memblock(void * blk)
+{
+	free(blk);
+}
