@@ -92,6 +92,9 @@ int right_connector_count(Disjunct *);
 Tracon_sharing *pack_sentence_for_pruning(Sentence);
 Tracon_sharing *pack_sentence_for_parsing(Sentence);
 void free_tracon_sharing(Tracon_sharing *);
+void free_tracon_memblock(Tracon_sharing *);
+void free_saved_memblock(void *);
+
 void count_disjuncts_and_connectors(Sentence, unsigned int *, unsigned int *);
 
 /* Print disjunct/connector */
