@@ -76,7 +76,7 @@ static const char *join_null_word(Sentence sent, Gword **wgp, size_t count)
 		join_len += strlen(wgp[i]->subword);
 
 	join_buff = alloca(join_len+1);
-	memset(join_buff, '\0', join_len+1);
+	join_buff[0] = 0;
 
 	for (i = 0; i < count; i++)
 		add_morpheme_unmarked(sent, join_buff, wgp[i]->subword,
@@ -101,13 +101,13 @@ static Gword *wordgraph_null_join(Sentence sent, Gword **start, Gword **end)
 	size_t join_len = 0;
 
 	for (w = start; w <= end; w++) join_len += strlen((*w)->subword);
-	usubword = calloc(join_len+1, 1); /* zeroed out */
+	usubword = alloca(join_len+1);
+	usubword[0] = 0;
 
 	for (w = start; w <= end; w++)
 		add_morpheme_unmarked(sent, usubword, (*w)->subword, (*w)->morpheme_type);
 
 	new_word = gword_new(sent, usubword);
-	free(usubword);
 	new_word->status |= WS_PL;
 	new_word->label = "NJ";
 	new_word->null_subwords = NULL;
@@ -534,8 +534,7 @@ static void compute_chosen_words(Sentence sent, Linkage linkage,
 						 * words with and without subscripts. */
 
 						const char subscript_sep_str[] = { SUBSCRIPT_SEP, '\0'};
-						char *join = calloc(join_len + 1, 1); /* zeroed out */
-
+						char *join = alloca(join_len + 1);
 						join[0] = '\0';
 
 						/* 1. Join base words. (Could just use the unsplit_word.) */
@@ -614,7 +613,6 @@ static void compute_chosen_words(Sentence sent, Linkage linkage,
 
 						gwordlist_append(&n_lwg_path, w->unsplit_word);
 						t = string_set_add(join, sent->string_set);
-						free(join);
 
 						i += mcnt-1;
 					}
@@ -694,12 +692,15 @@ static void compute_chosen_words(Sentence sent, Linkage linkage,
 	 * to facilitate using diff on sentence batch runs. */
 	if (test_enabled("removeZZZ"))
 	{
-		for (i=0; i<linkage->num_links; i++)
+		if (sent->dict->zzz_connector)
 		{
-			Link *lnk = &(linkage->link_array[i]);
+			for (i=0; i<linkage->num_links; i++)
+			{
+				Link *lnk = &(linkage->link_array[i]);
 
-			if (0 == strcmp("ZZZ", lnk->link_name))
-				chosen_words[lnk->rw] = NULL;
+				if (0 == strcmp(sent->dict->zzz_connector, lnk->link_name))
+					chosen_words[lnk->rw] = NULL;
+			}
 		}
 	}
 
@@ -768,7 +769,7 @@ void compute_generated_words(Sentence sent, Linkage linkage)
 		}
 		else
 		{
-			assert(cdj->num_categories > 0, "0 categories in disjunct");
+			assert(cdj->num_categories > 0, "zero categories in disjunct");
 			word = linkage_get_disjunct_str(linkage, i);
 			size_t len = strlen(word) + sizeof("<>");
 			char *disjunct_string = alloca(len);
@@ -877,20 +878,16 @@ const char ** linkage_get_words(const Linkage linkage)
 
 const char * linkage_get_disjunct_str(const Linkage linkage, WordIdx w)
 {
-	Disjunct *dj;
-
 	if (NULL == linkage) return "";
-	if (NULL == linkage->disjunct_list_str)
-	{
-		lg_compute_disjunct_strings(linkage);
-	}
-
 	if (linkage->num_words <= w) return NULL; /* bounds-check */
 
 	/* dj will be null if the word wasn't used in the parse. */
-	dj = linkage->chosen_disjuncts[w];
+	Disjunct *dj = linkage->chosen_disjuncts[w];
 	if (NULL == dj) return "";
 
+	if (NULL == linkage->disjunct_list_str)
+		lg_compute_disjunct_strings(linkage);
+
 	return linkage->disjunct_list_str[w];
 }
 
