@@ -14,9 +14,9 @@
 
 #include <limits.h>
 #include <inttypes.h>                   // PRIu64
-#if HAVE_THREADS_H
+#if HAVE_THREADS_H && !__EMSCRIPTEN__
 #include <threads.h>
-#endif /* HAVE_THREADS_H */
+#endif /* HAVE_THREADS_H && !__EMSCRIPTEN__ */
 
 #include "link-includes.h"
 #include "api-structures.h"
@@ -33,19 +33,19 @@
 
 #define D_COUNT 5 /* General debug level for this file. */
 
-typedef struct Table_connector_s Table_connector;
-struct Table_connector_s
-{
-	Table_connector  *next;
-	int              l_id, r_id;
-	Count_bin        count;
-	unsigned int     null_count;
-	size_t           hash;
-};
-
 typedef uint8_t null_count_m;  /* Storage representation of null_count */
 typedef uint8_t WordIdx_m;     /* Storage representation of word index */
 
+typedef struct Table_tracon_s Table_tracon;
+struct Table_tracon_s
+{
+	Table_tracon     *next;
+	int              l_id, r_id;
+	Count_bin        count;      // normally int_32_t
+	null_count_m     null_count;
+	size_t           hash;       // Generation needs more than 32 bits.
+};
+
 /* Most of the time, do_count() yields a zero leftcount/rightcount when it
  * parses a word range in which one end is a certain tracon and the other
  * end is a word that is between the nearest_word and farthest_word of the
@@ -68,15 +68,15 @@ typedef uint8_t WordIdx_m;     /* Storage representation of word index */
  * The check_next filed indicates the next word to be checked. Each
  * word-vector is updated each time one of its elements gets updated, so
  * check_next skips all the words for which the count is known to be zero. */
+
 typedef struct
 {
 	/* Arrays of match list information for sentences with null_count==0.
 	 * The count at index x is due to the disjuncts at the same index x. */
-	Disjunct **d_lkg_nc0;    /* Disjuncts with a jet linkage */
-	count_t *count_nc0;      /* The counts for that linkage */
+	match_list_cache *mlc0;
 
 	null_count_m null_count; /* status==0 valid up to this null count */
-	int8_t status;         /* -1: Needs update; 0: No count; 1: Count possible */
+	int8_t status;           /* -1: Needs update; 0: No count; 1: Count possible */
 	WordIdx_m check_next;    /* Next word to check */
 } count_expectation;
 
@@ -111,20 +111,52 @@ struct count_context_s
 	bool    exhausted;
 	uint8_t num_growth;       /* Number of table growths, for debug */
 	bool    is_short;
-	uint8_t log2_table_size;
-	uint32_t checktimer;  /* Avoid excess system calls */
-	size_t table_size;
-	size_t table_mask;
-	size_t table_available_count;
-	Table_connector ** table;
+	uint32_t checktimer;      /* Avoid excess system calls */
+	size_t table_size;        /* Can exceed 2**32 during generation. */
+	size_t table_mask;        /* 2**table_size -1 */
+	size_t table_available_count; /* derated table_size by hash load factor */
+	Table_tracon ** table;
 	Table_lrcnt table_lrcnt[2];  /* Left/right wordvec */
+	Pool_desc *mlc_pool;         /* Match list cache */
 	Resources current_resources;
 	COUNT_COST(uint64_t count_cost[3];)
 };
-#define MAX_TABLE_SIZE(s) (s / 10) /* Low load factor, for speed */
+
+/* Wikipedia claims that load factors of 0.6 to 0.75 are OK, and that
+ * a load factor of less than 0.25 is pointless. We set it to 0.333.
+ */
+#define INV_LOAD_FACTOR 3 /* One divided by load factor. */
+
+/* Avoid pathological cases leading to failure */
 #define MAX_LOG2_TABLE_SIZE ((sizeof(size_t)==4) ? 25 : 34)
 
-#if HAVE_THREADS_H
+/**
+ * Provide an estimate for the number of Table_tracon entries that will
+ * be needed.
+ *
+ * The number of entries actually used was measured in discussion
+ *     https://github.com/opencog/link-grammar/discussions/1402
+ * Based on this, an upper bound on the entries needed is
+ *    3 * num_disjuncts * log_2(num_words)
+ * i.e. more than this is almost never needed. A lower bound is
+ *    0.5 * num_disjuncts * log_2(num_words)
+ * i.e. more than this is *always* needed.
+ *
+ * In both conventional and MST dictionaries, more than 500K entries is
+ * almost never needed. In a handful of extreme cases, 2M was observed.
+ */
+static size_t estimate_tracon_entries(Sentence sent)
+{
+	unsigned int nwords = sent->length;
+	unsigned int log2_nwords = 0;
+	while (nwords) { log2_nwords++; nwords >>= 1; }
+
+	size_t tblsize = 3 * log2_nwords * sent->num_disjuncts;
+	if (tblsize < 512) tblsize = 512; // Happens rarely on short sentences.
+	return tblsize;
+}
+
+#if HAVE_THREADS_H && !__EMSCRIPTEN__
 /* Each thread will get it's own version of the `kept_table`.
  * If the program creates zillions of threads, then there will
  * be a mem-leak if this table is not released when each thread
@@ -135,11 +167,11 @@ static void free_tls_table(void* ptr_to_table)
 {
 	if (NULL == ptr_to_table) return;
 
-	Table_connector** kept_table = *((Table_connector***)ptr_to_table);
+	Table_tracon** kept_table = *((Table_tracon***)ptr_to_table);
 	if (NULL == kept_table) return;
 
 	free(kept_table);
-	*((Table_connector***) ptr_to_table) = NULL;
+	*((Table_tracon***) ptr_to_table) = NULL;
 }
 
 static tss_t key;
@@ -147,93 +179,91 @@ static void make_key(void)
 {
 	tss_create(&key, free_tls_table);
 }
-#endif /* HAVE_THREADS_H */
+#endif /* HAVE_THREADS_H && !__EMSCRIPTEN__ */
 
 /**
- * Allocate memory for the connector-pair table and initialize table-size
- * related fields (table_size and table_available_count). Reuse the
- * previous table memory if the request is for a table that fits. If this
- * is not a call to grow the table, free it if not reused.
+ * Allocate memory for the connector-pair hash table and initialize
+ * table-size related fields (table_size and table_available_count).
+ * Reuse the previous hash table memory if the request is for a table
+ * that fits.
  *
  * @ctxt[in, out] Table info.
- * @param shift log2 table size, or \c 0 for table growth. On table growth,
- * increase the new table size by 2.
+ * @param logsz Log2 requested table size, or \c 0 if the table needs
+ *              to be expanded. Tables are expanded by a factor of 2.
  */
-static void table_alloc(count_context_t *ctxt, unsigned int shift)
+static void table_alloc(count_context_t *ctxt, unsigned int logsz)
 {
-	static TLS Table_connector **kept_table = NULL;
-	static TLS unsigned int log2_kept_table_size = 0;
+	static TLS Table_tracon **kept_table = NULL;
+	static TLS size_t kept_table_size = 0;
 
-#if HAVE_THREADS_H
+	size_t reqsz = 1ULL << logsz;
+	if (0 < logsz && reqsz <= ctxt->table_size) return; // It's big enough, already.
+
+#if HAVE_THREADS_H && !__EMSCRIPTEN__
 	// Install a thread-exit handler, to free kept_table on thread-exit.
 	static once_flag flag = ONCE_FLAG_INIT;
 	call_once(&flag, make_key);
 
 	if (NULL == kept_table)
 		tss_set(key, &kept_table);
-#endif /* HAVE_THREADS_H */
+#endif /* HAVE_THREADS_H && !__EMSCRIPTEN__ */
 
-	if (shift == 0)
-		shift = ctxt->log2_table_size + 1; /* Double the table size */
-	lgdebug(+D_COUNT, "Connector table log2 size %u\n", shift);
+	if (logsz == 0)
+		ctxt->table_size *= 2; /* Double the table size */
+	else
+		ctxt->table_size = reqsz;
+
+	if ((1ULL << MAX_LOG2_TABLE_SIZE) <= ctxt->table_size)
+		ctxt->table_size  = (1ULL << MAX_LOG2_TABLE_SIZE);
+
+	lgdebug(+D_COUNT, "Tracon table size %lu\n", ctxt->table_size);
 
 	/* Keep the table indefinitely (until thread-exit), so that it can
 	 * be reused. This avoids a large overhead in malloc/free when
-	 * large memory blocks are allocated. Large block in Linux trigger
+	 * large memory blocks are allocated. Large blocks in Linux trigger
 	 * system calls to mmap/munmap that eat up a lot of time.
-	 * (Up to 20%, depending on the sentence and CPU.) */
-	ctxt->table_size = (1ULL << shift);
-	if ((shift > log2_kept_table_size) || (kept_table == NULL))
+	 * (Up to 20%, depending on the sentence and CPU.)
+	 *
+	 * FYI: the new tracon tables are (much?) smaller than the older
+	 * connector tables, so maybe this reuse is no longer needed?
+	 */
+	if (kept_table_size < ctxt->table_size)
 	{
-		log2_kept_table_size = shift;
+		kept_table_size = ctxt->table_size;
 
 		if (kept_table) free(kept_table);
-		kept_table = malloc(sizeof(Table_connector *) * ctxt->table_size);
+		kept_table = malloc(sizeof(Table_tracon *) * ctxt->table_size);
 	}
-
-	memset(kept_table, 0, sizeof(Table_connector *) * ctxt->table_size);
-
-	/* This is here and not in init_table() because it must be set
-	 * also when the table growths. */
-	ctxt->log2_table_size = shift;
-	ctxt->table_mask = ctxt->table_size - 1;
 	ctxt->table = kept_table;
 
-	if (shift >= MAX_LOG2_TABLE_SIZE)
-		ctxt->table_available_count = UINT_MAX; /* Prevent growth */
-	else
-		ctxt->table_available_count = MAX_TABLE_SIZE(ctxt->table_size);
+	memset(ctxt->table, 0, sizeof(Table_tracon *) * ctxt->table_size);
+
+	ctxt->table_mask = ctxt->table_size - 1;
+
+	// This assures that the table load will stay under the load factor.
+	ctxt->table_available_count = ctxt->table_size / INV_LOAD_FACTOR;
 }
 
 /**
- * Allocate the table with a sentence-dependent table size.  Use
- * sent->length as a hint for the initial table size. Usually, this
- * saves on dynamic table growth, which is costly.
- * */
+ * Allocate the tracon hash table with a sentence-dependent table size.
+ * This saves on dynamic table growth, which is costly.
+ *
+ * We estimate the number of required hash table slots by estimating
+ * the number of entries that will be required, and then multiplying by
+ * INV_LOAD_FACTOR so that the hash table usage remains sparse.
+ */
 static void init_table(count_context_t *ctxt)
 {
-	Sentence sent = ctxt->sent;
+	// Number of tracon entries we expect to store.
+	size_t tblsz = estimate_tracon_entries(ctxt->sent);
 
-	/* A piecewise exponential function determines the size of the
-	 * hash table. Probably should make use of the actual number of
-	 * disjuncts, rather than just the number of words.
-	 */
-	unsigned int shift;
-	if (sent->length >= 16)
-	{
-		shift = 14 + sent->length / 16;
-	}
-	else
-	{
-		shift = 14;
-	}
-#if 0 /* Not yet */
-	shift += (unsigned int)sent->num_disjuncts / (sent->length * 200);
-#endif
+	// Adjust by the table load factor.
+	tblsz *= INV_LOAD_FACTOR;
 
-	if (MAX_LOG2_TABLE_SIZE < shift) shift = MAX_LOG2_TABLE_SIZE;
+	unsigned int logsz = 0;
+	while (tblsz) { logsz++; tblsz >>= 1; }
 
-	table_alloc(ctxt, shift);
+	table_alloc(ctxt, logsz);
 }
 
 static void free_table_lrcnt(count_context_t *ctxt)
@@ -243,7 +273,7 @@ static void free_table_lrcnt(count_context_t *ctxt)
 	if (verbosity_level(D_COUNT))
 	{
 		unsigned int nonzero = 0, any_null = 0, zero = 0, non_max_null = 0;
-		unsigned int cml = 0, cml_disjunct = 0; /* Cashed match lists */
+		unsigned int cml = 0, cml_disjunct = 0; /* Cached match lists */
 		Pool_location loc = { 0 };
 		wordvecp t;
 
@@ -256,10 +286,10 @@ static void free_table_lrcnt(count_context_t *ctxt)
 			if (t->status == 1)
 			{
 				nonzero++;
-				if (t->d_lkg_nc0 && (t->d_lkg_nc0 != NULL))
+				if (t->mlc0 != NULL)
 				{
 					cml++;
-					for (Disjunct **d = t->d_lkg_nc0; *d != NULL; d++)
+					for (Disjunct *d = t->mlc0->d_lkg; d != NULL; d++)
 						cml_disjunct++;
 				}
 			}
@@ -271,7 +301,8 @@ static void free_table_lrcnt(count_context_t *ctxt)
 				zero++;
 		}
 
-		const unsigned int num_values = ctxt->sent->wordvec_pool->curr_elements;
+		const unsigned int num_values =
+			pool_num_elements_issued(ctxt->sent->wordvec_pool);
 		lgdebug(+0, "Values %u (usage = non_max_null %u + other %u, "
 		        "other = any_null_zero %u + zero %u + nonzero %u); "
 		        "%u disjuncts in %u cache entries\n",
@@ -294,6 +325,8 @@ static void free_table_lrcnt(count_context_t *ctxt)
 		}
 	}
 
+	pool_delete(ctxt->mlc_pool);
+
 	for (unsigned int dir = 0; dir < 2; dir++)
 	{
 		free(ctxt->table_lrcnt[dir].tracon_wvp);
@@ -301,6 +334,40 @@ static void free_table_lrcnt(count_context_t *ctxt)
 	}
 }
 
+/// Estimate the proper size of the match pool based on experimental
+/// data. An upper bound of twice the number of expressions seems to
+/// handle almost all cases. This is based on the graph in
+/// https://github.com/opencog/link-grammar/discussions/1402#discussioncomment-4826342
+/// The estimate is meant to be an *upper bound* for how many will be
+/// needed; the goal is to avoid allocation to get more, because it is
+/// expensive.
+///
+/// FYI, Expression pool sizes in excess of 10M entries have been observed.
+static size_t match_list_pool_size_estimate(Sentence sent)
+{
+	size_t expsz = pool_num_elements_issued(sent->Exp_pool);
+
+	size_t mlpse = 2 * expsz;
+	if (mlpse < 4090) mlpse = 4090;
+
+	// Code below does a pool_alloc_vec(match_list_size) and we want to
+	// ensure that this estimate is greater than the match list size.
+	// The match list size can never be larger than the number of
+	// disjuncts, so the pool_alloc_vec() will never fail if we do this:
+	size_t maxndj = 0;
+	for (WordIdx w = 0; w < sent->length; w++)
+		if (maxndj < sent->word[w].num_disjuncts)
+			maxndj = sent->word[w].num_disjuncts;
+
+	// Generation can have millions of disjuncts on the wild-cards.
+	// But the match list will never get that big.
+	if (512*1024 < maxndj) maxndj = 512*1024;
+
+	if (mlpse < maxndj) mlpse = maxndj;
+
+	return mlpse;
+}
+
 static void init_table_lrcnt(count_context_t *ctxt)
 {
 	Sentence sent = ctxt->sent;
@@ -326,6 +393,14 @@ static void init_table_lrcnt(count_context_t *ctxt)
 			         sizeof(count_expectation), /*zero_out*/true,
 			         /*align*/false, /*exact*/false);
 	}
+
+	const size_t match_list_pool_size = match_list_pool_size_estimate(sent);
+
+	/* FIXME: Modify pool_alloc_vec() to use dynamic block sizes. */
+	ctxt->mlc_pool =
+		pool_new(__func__, "Match list cache",
+		         /*num_elements*/match_list_pool_size, sizeof(match_list_cache),
+		         /*zero_out*/false, /*align*/false, /*exact*/false);
 }
 
 #ifdef DEBUG
@@ -357,7 +432,7 @@ static void table_stat(count_context_t *ctxt)
 
 	for (size_t i = 0; i < ctxt->table_size; i++)
 	{
-		Table_connector *t = ctxt->table[i];
+		Table_tracon *t = ctxt->table[i];
 
 		c = 0;
 		if (t == NULL)
@@ -396,11 +471,14 @@ static void table_stat(count_context_t *ctxt)
 	}
 
 	size_t used_slots = ctxt->table_size-N;
+	unsigned int logsz = 0;
+	size_t tblsz = ctxt->table_size;
+	while (tblsz) { logsz++; tblsz >>= 1; }
 	/* The used= value is TotalValues/TableSize (not UsedSlots/TableSize). */
 	printf("Connector table: num_growth=%u msb=%u slots=%6zu/%6zu (%5.2f%%) "
 	       "avg-chain=%4.2f values=%6zu (z=%5zu nz=%5zu N=%5zu) used=%5.2f%% "
 	       "acc=%"PRIu64" (hit=%"PRIu64" miss=%"PRIu64") (sent_len=%zu dis=%u)\n",
-	       ctxt->num_growth, ctxt->log2_table_size, used_slots, ctxt->table_size,
+	       ctxt->num_growth, logsz, used_slots, ctxt->table_size,
 			 100.0f*used_slots/ctxt->table_size, 1.0f*total_c/used_slots,
 	       z+nz, z, nz, N, 100.0f*(z+nz)/ctxt->table_size,
 	       hit+miss, hit, miss, ctxt->sent->length, ctxt->sent->num_disjuncts);
@@ -429,7 +507,7 @@ static void table_stat(count_context_t *ctxt)
 			printf("Null count %u:\n", nc);
 			for (size_t i = 0; i < ctxt->table_size; i++)
 			{
-				for (Table_connector *t = ctxt->table[i]; t != NULL; t = t->next)
+				for (Table_tracon *t = ctxt->table[i]; t != NULL; t = t->next)
 				{
 					if (t->null_count != nc) continue;
 
@@ -446,12 +524,20 @@ static void table_stat(count_context_t *ctxt)
 
 static void table_grow(count_context_t *ctxt)
 {
+	// If we somehow hit the max size, disallow further growth.
+	// Reset the `table_available_count` so we don't come here again.
+	if ((1ULL << MAX_LOG2_TABLE_SIZE) <= ctxt->table_size)
+	{
+		ctxt->table_available_count = UINT_MAX;
+		return;
+	}
+
 	table_alloc(ctxt, 0);
 
 	/* Rehash. */
-	Table_connector *oe;
+	Table_tracon *oe;
 	Pool_location loc = { 0 };
-	while ((oe = pool_next(ctxt->sent->Table_connector_pool, &loc)) != NULL)
+	while ((oe = pool_next(ctxt->sent->Table_tracon_pool, &loc)) != NULL)
 	{
 		size_t ni = oe->hash & ctxt->table_mask;
 
@@ -477,7 +563,7 @@ static Count_bin table_store(count_context_t *ctxt,
 	int l_id = (NULL != le) ? le->tracon_id : lw;
 	int r_id = (NULL != re) ? re->tracon_id : rw;
 	size_t i = hash & ctxt->table_mask;
-	Table_connector *n = pool_alloc(ctxt->sent->Table_connector_pool);
+	Table_tracon *n = pool_alloc(ctxt->sent->Table_tracon_pool);
 
 	if (ctxt->table[i] == NULL)
 		ctxt->table_available_count--;
@@ -509,7 +595,7 @@ table_lookup(count_context_t *ctxt, int lw, int rw,
 	int r_id = (NULL != re) ? re->tracon_id : rw;
 
 	size_t h = pair_hash(lw, rw, l_id, r_id, null_count);
-	Table_connector *t = ctxt->table[h & ctxt->table_mask];
+	Table_tracon *t = ctxt->table[h & ctxt->table_mask];
 
 	for (; t != NULL; t = t->next)
 	{
@@ -645,11 +731,12 @@ static void lrcnt_keep_count(wordvecp lrcnt_cache, bool dir, Disjunct *d,
 /**
  * Cache the match list elements that yield a non-zero count.
  */
-static void lrcnt_cache_match_list(wordvecp lrcnt_cache, fast_matcher_t *mchxt,
+static void lrcnt_cache_match_list(wordvecp lrcnt_cache, count_context_t *ctxt,
                                    size_t mlb, bool dir)
 {
 	size_t dcnt = 0;
 	size_t i  = 0;
+	fast_matcher_t *mchxt = ctxt->mchxt;
 
 	for (i = mlb; get_match_list_element(mchxt, i) != NULL; i++)
 	{
@@ -661,10 +748,8 @@ static void lrcnt_cache_match_list(wordvecp lrcnt_cache, fast_matcher_t *mchxt,
 	lgdebug(+9, "MATCH_LIST %9d dir=%d mlb %zu cached %zu/%zu\n",
 	        get_match_list_element(mchxt, mlb)->match_id, dir, mlb, dcnt, i-mlb);
 
-	Disjunct **ml = pool_alloc_vec(mchxt->mld_pool, dcnt + 1);
-	count_t *c = pool_alloc_vec(mchxt->mlc_pool, dcnt);
-
-	if ((ml == NULL) || (c == NULL)) return; /* Cannot allocate cache array */
+	match_list_cache *ml = pool_alloc_vec(ctxt->mlc_pool, dcnt + 1);
+	if (ml == NULL) return; /* Cannot allocate cache array */
 
 	dcnt = 0;
 	for (i = mlb; get_match_list_element(mchxt, i) != NULL; i++)
@@ -672,16 +757,15 @@ static void lrcnt_cache_match_list(wordvecp lrcnt_cache, fast_matcher_t *mchxt,
 		Disjunct *d = get_match_list_element(mchxt, i);
 		if ((dir == 0) ? d->match_left : d->match_right)
 		{
-			ml[dcnt] = d;
+			ml[dcnt].d_lkg = d;
 			assert(d->lrcount > 0, "Invalid linkage count %d", d->lrcount);
-			c[dcnt] = d->lrcount;
+			ml[dcnt].count = d->lrcount;
 			dcnt++;
 		}
 	}
-	ml[dcnt] = NULL;
+	ml[dcnt].d_lkg = NULL;
 
-	lrcnt_cache->d_lkg_nc0 = ml;
-	lrcnt_cache->count_nc0 = c;
+	lrcnt_cache->mlc0 = ml;
 }
 
 /**
@@ -779,8 +863,8 @@ bool no_count(count_context_t *ctxt, int dir, Connector *c,
 	return (lrcnt_check(lrcnt_cache, null_count, NULL) == &lrcnt_cache_zero);
 }
 
-Disjunct ***get_cached_match_list(count_context_t *ctxt, int dir, int w,
-                                  Connector *c)
+match_list_cache *get_cached_match_list(count_context_t *ctxt, int dir, int w,
+                                        Connector *c)
 {
 	if (ctxt->sent->null_count != 0) return NULL;
 	if (ctxt->is_short) return NULL;
@@ -788,22 +872,7 @@ Disjunct ***get_cached_match_list(count_context_t *ctxt, int dir, int w,
 	wordvecp wv = ctxt->table_lrcnt[dir].tracon_wvp[c->tracon_id];
 	if (wv == NULL) return NULL;
 
-	return &wv[w - ((dir == 0) ? c->nearest_word : c->farthest_word)].d_lkg_nc0;
-}
-
-static Count_bin *get_cached_count(count_context_t *ctxt, int dir, int w,
-                                    Connector *c)
-{
-	if (ctxt->sent->null_count != 0) return NULL;
-
-	wordvecp wv = ctxt->table_lrcnt[dir].tracon_wvp[c->tracon_id];
-	if (wv == NULL) return NULL;
-
-	Count_bin *count =
-		wv[w - ((dir == 0) ? c->nearest_word : c->farthest_word)].count_nc0;
-	if (count == NULL) return NULL;
-
-	return count;
+	return wv[w - ((dir == 0) ? c->nearest_word : c->farthest_word)].mlc0;
 }
 
 static bool lrcnt_expectation_update(wordvecp wv, bool lrcnt_found,
@@ -863,7 +932,7 @@ static Count_bin pseudocount(count_context_t * ctxt,
                            unsigned int null_count)
 {
 	/* This check is not necessary for correctness, but it saves CPU time.
-	 * If a cross link would result, immediatly return 0. Note that there is
+	 * If a cross link would result, immediately return 0. Note that there is
 	 * no need to check here if the nearest_word fields are in the range
 	 * [lw, rw] due to the way start_word/end_word are computed, and due to
 	 * nearest_word checks in form_match_list(). */
@@ -996,7 +1065,7 @@ static Count_bin do_count(
 	int start_word, end_word, w;
 
 	/* This check is not necessary for correctness. It typically saves
-	 * significant CPU and Table_connector memory because in many cases a
+	 * significant CPU and Table_tracon memory because in many cases a
 	 * linkage is not possible due to nearest_word restrictions. */
 	if (!valid_nearest_words(le, re, lw, rw)) return hist_zero();
 
@@ -1170,8 +1239,8 @@ static Count_bin do_count(
 		 * extremely effective for long sentences, but doesn't speed up
 		 * short ones.
 		 *
-		 * FIXME: l/rcnt_optimize==false doubles the Table_connector cache!
-		 * Try to fix it by not caching zero counts in Table_connector when
+		 * FIXME: l/rcnt_optimize==false doubles the Table_tracon cache!
+		 * Try to fix it by not caching zero counts in Table_tracon when
 		 * l/rcnt_optimize==false. If this can be fixed, a significant
 		 * speedup is expected. */
 
@@ -1182,10 +1251,8 @@ static Count_bin do_count(
 		unsigned int lnull_start = 0; /* First null_count to check */
 		unsigned int lnull_end = null_count; /* Last null_count to check */
 		Connector *fml_re = re;       /* For form_match_list() only */
-		Disjunct ***mlcl = NULL, ***mlcr = NULL;
+		match_list_cache *mlcl = NULL, *mlcr = NULL;
 		bool using_cached_match_list = false;
-		Count_bin *l_cache = NULL;
-		Count_bin *r_cache = NULL;
 		unsigned int lcount_index = 0; /* Cached left count index */
 
 		if (ctxt->is_short)
@@ -1242,12 +1309,10 @@ static Count_bin do_count(
 
 				if (le != NULL)
 				{
-					l_cache = get_cached_count(ctxt, 0, w, le);
 					mlcl = get_cached_match_list(ctxt, 0, w, le);
 				}
 				if (fml_re != NULL && ((le == NULL) || (re->farthest_word <= w)))
 				{
-					r_cache = get_cached_count(ctxt, 1, w, re);
 					mlcr = get_cached_match_list(ctxt, 1, w, re);
 				}
 			}
@@ -1289,15 +1354,15 @@ static Count_bin do_count(
 				 * but for the cached right count we depend on form_match_list()
 				 * to set d->rcount_index as the index into the corresponding
 				 * cached right count array. */
-				if (Lmatch && (l_cache != NULL))
+				if (Lmatch && (mlcl != NULL))
 				{
 					leftpcount = true;
-					leftcount = l_cache[lcount_index++];
+					leftcount = mlcl[lcount_index++].count;
 				}
-				if (Rmatch && (r_cache != NULL))
+				if (Rmatch && (mlcr != NULL))
 				{
 					rightpcount = true;
-					rightcount = r_cache[d->rcount_index];
+					rightcount = mlcr[d->rcount_index].count;
 				}
 			}
 
@@ -1515,7 +1580,7 @@ static Count_bin do_count(
 				lrcnt_cache_changed = true;
 			}
 			if (lrcnt_found && (ctxt->sent->null_count == 0))
-				lrcnt_cache_match_list(lrcnt_cache, mchxt, mlb, /*dir*/le == NULL);
+				lrcnt_cache_match_list(lrcnt_cache, ctxt, mlb, /*dir*/le == NULL);
 		}
 
 		pop_match_list(mchxt, mlb);
@@ -1596,15 +1661,27 @@ count_context_t * alloc_count_context(Sentence sent, Tracon_sharing *ts)
 	 * one null link. */
 	/* ctxt->null_block = 1; */
 
-	if (NULL != sent->Table_connector_pool)
+	if (NULL != sent->Table_tracon_pool)
 	{
-		pool_reuse(sent->Table_connector_pool);
+		pool_reuse(sent->Table_tracon_pool);
 	}
 	else
 	{
-		sent->Table_connector_pool =
-			pool_new(__func__, "Table_connector",
-			         /*num_elements*/16384, sizeof(Table_connector),
+		/* The estimate_tracon_entries() provides an upper limit to
+		 * how many might get allocated in a "typical" sentence. The
+		 * lower bound is 12 times smaller. So, start with that. If
+		 * this isn't enough, each new alloc will grab another chunk
+		 * of this size. Thus, a typical sentence will expand the
+		 * tracon pool six times. But this logic holds only if a new
+		 * pool is allocated each time. When re-using, with sentences
+		 * of highly variable sizes, if seems best to fix this value.
+		 *
+		 * size_t num_elts = estimate_tracon_entries(sent);
+		 * num_elts /= 12;
+		 */
+		sent->Table_tracon_pool =
+			pool_new(__func__, "Table_tracon",
+			         16382 /* num_elts */, sizeof(Table_tracon),
 			         /*zero_out*/false, /*align*/false, /*exact*/false);
 	}
 
