@@ -94,12 +94,10 @@ void free_fast_matcher(Sentence sent, fast_matcher_t *mchxt)
 	if (NULL == mchxt) return;
 
 	free(mchxt->l_table[0]);
-	free(mchxt->match_list);
+	xfree(mchxt->match_list, mchxt->match_list_size * sizeof(*mchxt->match_list));
 	lgdebug(+6, "Sentence length %zu, match_list_size %zu\n",
 	        mchxt->size, mchxt->match_list_size);
 
-	pool_delete(mchxt->mld_pool);
-	pool_delete(mchxt->mlc_pool);
 	xfree(mchxt->l_table_size, mchxt->size * sizeof(unsigned int));
 	xfree(mchxt->l_table, mchxt->size * sizeof(Match_node **));
 	xfree(mchxt, sizeof(fast_matcher_t));
@@ -259,16 +257,6 @@ fast_matcher_t* alloc_fast_matcher(const Sentence sent, unsigned int *ncu[])
 			         /*num_elements*/2048, sizeof(Match_node),
 			         /*zero_out*/false, /*align*/true, /*exact*/false);
 	}
-	const size_t match_list_pool_size = 512*1024; /* Currently a hard limit. */
-	/* FIXME: Modify pool_alloc_vec() to use dynamic block sizes. */
-	ctxt->mld_pool =
-		pool_new(__func__, "Match list cache",
-		         /*num_elements*/match_list_pool_size, sizeof(Disjunct *),
-		         /*zero_out*/false, /*align*/false, /*exact*/false);
-	ctxt->mlc_pool =
-		pool_new(__func__, "Match list counts",
-		         /*num_elements*/match_list_pool_size, sizeof(count_t),
-		         /*zero_out*/false, /*align*/false, /*exact*/false);
 
 	sortbin *sbin = alloca(sent->length * sizeof(sortbin));
 
@@ -413,7 +401,8 @@ static void match_stats(Connector *c1, Connector *c2)
 static void print_match_list(fast_matcher_t *ctxt, uint16_t id, size_t mlb, int w,
                              Connector *lc, int lw,
                              Connector *rc, int rw,
-                             Disjunct ***mlcl, Disjunct ***mlcr)
+                             match_list_cache *mlcl,
+                             match_list_cache *mlcr)
 {
 	if (!verbosity_level(D_FAST_MATCHER)) return;
 	Disjunct **m = &ctxt->match_list[mlb];
@@ -533,10 +522,11 @@ static bool alt_connection_possible(Connector *c1, Connector *c2,
  * Return the match list start.
  */
 static size_t terminate_match_list(fast_matcher_t *ctxt, uint16_t id,
-                             size_t ml_start, int w,
-                             Connector *lc, int lw,
-                             Connector *rc, int rw,
-                             Disjunct ***mlcl, Disjunct ***mlcr)
+                                   size_t ml_start, int w,
+                                   Connector *lc, int lw,
+                                   Connector *rc, int rw,
+                                   match_list_cache *mlcl,
+                                   match_list_cache *mlcr)
 {
 	push_match_list_element(ctxt, 0, NULL);
 	print_match_list(ctxt, id, ml_start, w, lc, lw, rc, rw, mlcl, mlcr);
@@ -570,18 +560,15 @@ size_t
 form_match_list(fast_matcher_t *ctxt, int w,
                 Connector *lc, int lw,
                 Connector *rc, int rw,
-                Disjunct ***mlcl, Disjunct ***mlcr)
+                match_list_cache *mlcl, match_list_cache *mlcr)
 {
 	Match_node *mx, *mr_end;
 	size_t front = get_match_list_position(ctxt);
 	Match_node *ml = NULL, *mr = NULL; /* Initialize in case of NULL lc or rc. */
-	Disjunct **cmx;
+	match_list_cache *cmx;
 	match_cache mc;
 	gword_cache gc = { .same_alternative = false };
 
-	if ((mlcl != NULL) && (*mlcl == NULL)) mlcl = NULL;
-	if ((mlcr != NULL) && (*mlcr == NULL)) mlcr = NULL;
-
 	if (mlcl == NULL)
 	{
 		/* Get the lists of candidate matching disjuncts of word w for lc and
@@ -615,7 +602,7 @@ form_match_list(fast_matcher_t *ctxt, int w,
 #endif
 
 	lgdebug(+D_FAST_MATCHER, "MATCH_LIST %c%c %5d mlb %zu\n",
-		       (mlcl == NULL) ? ' ' : 'L', (mlcr == NULL) ? ' ' : 'R', id, front);
+	        (mlcl == NULL) ? ' ' : 'L', (mlcr == NULL) ? ' ' : 'R', id, front);
 
 	if (mlcr == NULL)
 	{
@@ -628,11 +615,11 @@ form_match_list(fast_matcher_t *ctxt, int w,
 	}
 	else
 	{
-		for (cmx = *mlcr; *cmx != NULL; cmx++)
+		for (cmx = mlcr; cmx->d_lkg != NULL; cmx++)
 		{
-			(*cmx)->match_left = false;
+			cmx->d_lkg->match_left = false;
 		}
-		mr_end = NULL; /* Prevent a gcc "mnay be uninitialkized" warning */
+		mr_end = NULL; /* Prevent a gcc "may be uninitialized" warning */
 	}
 
 	/* Construct the list of things that could match the left. */
@@ -659,11 +646,11 @@ form_match_list(fast_matcher_t *ctxt, int w,
 	}
 	else
 	{
-		for (cmx = *mlcl; *cmx != NULL; cmx++)
+		for (cmx = mlcl; cmx->d_lkg != NULL; cmx++)
 		{
-			(*cmx)->match_left = true;
-			(*cmx)->match_right = false;
-			push_match_list_element(ctxt, lid, *cmx);
+			cmx->d_lkg->match_left = true;
+			cmx->d_lkg->match_right = false;
+			push_match_list_element(ctxt, lid, cmx->d_lkg);
 		}
 	}
 
@@ -690,13 +677,13 @@ form_match_list(fast_matcher_t *ctxt, int w,
 	}
 	else
 	{
-		for (cmx = *mlcr; *cmx != NULL; cmx++)
+		for (cmx = mlcr; cmx->d_lkg != NULL; cmx++)
 		{
-			if ((lc != NULL) && !(*cmx)->match_left) continue; /* lc optimization*/
-			(*cmx)->match_right = true;
-			(*cmx)->rcount_index = (uint32_t)(cmx - *mlcr);
-			if ((*cmx)->match_left) continue;
-			push_match_list_element(ctxt, lid, *cmx);
+			if ((lc != NULL) && !cmx->d_lkg->match_left) continue; /* lc optimization*/
+			cmx->d_lkg->match_right = true;
+			cmx->d_lkg->rcount_index = (uint32_t)(cmx - mlcr);
+			if (cmx->d_lkg->match_left) continue;
+			push_match_list_element(ctxt, lid, cmx->d_lkg);
 		}
 	}
 
