@@ -20,6 +20,12 @@
 #include "link-includes.h"              // for Sentence
 #include "memory-pool.h"
 
+typedef struct match_list_cache_sruct
+{
+	Disjunct *d_lkg;        /* Disjuncts with a jet linkage */
+	Count_bin count;          /* The counts for that linkage */
+} match_list_cache;
+
 typedef struct Match_node_struct Match_node;
 struct Match_node_struct
 {
@@ -42,10 +48,6 @@ struct fast_matcher_s
 	Disjunct ** match_list;      /* match-list stack */
 	size_t match_list_end;       /* index to the match-list stack end */
 	size_t match_list_size;      /* number of allocated elements */
-
-	/* Match list cache. */
-	Pool_desc *mld_pool; /* disjuncts pointers */
-	Pool_desc *mlc_pool; /* linkage counts */
 };
 
 /* See the source file for documentation. */
@@ -53,7 +55,7 @@ fast_matcher_t* alloc_fast_matcher(const Sentence, unsigned int *[]);
 void free_fast_matcher(Sentence sent, fast_matcher_t*);
 
 size_t form_match_list(fast_matcher_t *, int, Connector *, int, Connector *,
-                       int, Disjunct ***, Disjunct ***);
+                       int, match_list_cache *, match_list_cache *);
 
 /**
  * Return the match-list element at the given index.
