@@ -53,7 +53,7 @@
 bool post_process_match(const char *s, const char *t)
 {
 	if (NULL == t) return false;
-	if (islower(*t)) t++; /* Skip head-dependent indicator */
+	if (islower((unsigned char)*t)) t++; /* Skip head-dependent indicator */
 	while (is_connector_name_char(*s))
 	{
 		if (*s != *t) return false;
