@@ -33,9 +33,8 @@ typedef struct clause_struct Clause;
 struct clause_struct
 {
 	Clause * next;
-	float cost;
-	float maxcost;
 	Tconnector * c;
+	float totcost;   // Total cost of all connectors in the clause
 };
 
 typedef struct
@@ -138,8 +137,7 @@ static Clause * build_clause(Exp *e, clause_context *ct, Clause **c_last)
 		Clause *c1 = pool_alloc(ct->Clause_pool);
 		c1->c = NULL;
 		c1->next = NULL;
-		c1->cost = 0.0;
-		c1->maxcost = 0.0;
+		c1->totcost = 0.0;
 		for (Exp *opd = e->operand_first; opd != NULL; opd = opd->operand_next)
 		{
 			Clause *c2 = build_clause(opd, ct, NULL);
@@ -148,14 +146,9 @@ static Clause * build_clause(Exp *e, clause_context *ct, Clause **c_last)
 			{
 				for (Clause *c4 = c2; c4 != NULL; c4 = c4->next)
 				{
-					float maxcost = MAX(c3->maxcost,c4->maxcost);
-					/* Cannot use this shortcut due to negative costs. */
-					//if (maxcost + e->cost > ct->cost_cutoff) continue;
-
 					Clause *c5 = pool_alloc(ct->Clause_pool);
 					if ((c_head == NULL) && (c_last != NULL)) *c_last = c5;
-					c5->cost = c3->cost + c4->cost;
-					c5->maxcost = maxcost;
+					c5->totcost = c3->totcost + c4->totcost;
 					c5->c = catenate(c4->c, c3->c, ct->Tconnector_pool);
 					c5->next = c_head;
 					c_head = c5;
@@ -191,8 +184,7 @@ static Clause * build_clause(Exp *e, clause_context *ct, Clause **c_last)
 	{
 		c = pool_alloc(ct->Clause_pool);
 		c->c = build_terminal(e, ct);
-		c->cost = 0.0;
-		c->maxcost = 0.0;
+		c->totcost = 0.0;
 		c->next = NULL;
 		if (c_last != NULL) *c_last = c;
 	}
@@ -203,19 +195,8 @@ static Clause * build_clause(Exp *e, clause_context *ct, Clause **c_last)
 
 	/* c now points to the list of clauses */
 	for (Clause *c1 = c; c1 != NULL; c1 = c1->next)
-	{
-		c1->cost += e->cost;
-		/* c1->maxcost = MAX(c1->maxcost,e->cost);  */
-		/* Above is how Dennis had it. Someone changed it to below.
-		 * However, this can sometimes lead to a maxcost that is less
-		 * than the cost ! -- which seems wrong to me ... seems Dennis
-		 * had it right!?
-		 */
-		c1->maxcost += e->cost;
-		/* Note: The above computation is used as a saving shortcut in
-		 * the inner loop of AND_type. If it is changed here, it needs to be
-		 * changed there too. */
-	}
+		c1->totcost += e->cost;
+
 	return c;
 }
 
@@ -227,31 +208,31 @@ static Disjunct *
 build_disjunct(Sentence sent, Clause * cl, const char * string,
                const gword_set *gs, float cost_cutoff, Parse_Options opts)
 {
-	Disjunct *dis, *ndis;
-	Pool_desc *connector_pool = NULL;
+	Pool_desc *connector_pool = sent->Connector_pool;
+	Pool_desc *disjunct_pool = sent->Disjunct_pool;
 	bool sat_solver = false;
 
 #if USE_SAT_SOLVER
-		sat_solver = (opts != NULL) && opts->use_sat_solver;
+	sat_solver = (opts != NULL) && opts->use_sat_solver;
 #endif /* USE_SAT_SOLVER */
 
-	dis = NULL;
+	Disjunct *dis = NULL;
 	for (; cl != NULL; cl = cl->next)
 	{
+		Disjunct *ndis;
+
 		if (unlikely(NULL == cl->c)) continue; /* no connectors */
-		if (cl->maxcost > cost_cutoff) continue;
+		if (cl->totcost > cost_cutoff) continue;
 
 #if USE_SAT_SOLVER
 		if (sat_solver) /* For the SAT-parser, until fixed. */
-		{
 			ndis = xalloc(sizeof(Disjunct));
-		}
 		else
+			ndis = pool_alloc(disjunct_pool);
+#else
+		ndis = pool_alloc(disjunct_pool);
 #endif
-		{
-			ndis = pool_alloc(sent->Disjunct_pool);
-			connector_pool = sent->Connector_pool;
-		}
+
 		ndis->left = ndis->right = NULL;
 
 		/* Build a list of connectors from the Tconnectors. */
@@ -264,7 +245,7 @@ build_disjunct(Sentence sent, Clause * cl, const char * string,
 			if (is_tracon[idir]) continue; /* this direction is complete */
 			if (NULL != t->tracon)
 			{
-				/* Use the cashed tracon and mark this direction as complete. */
+				/* Use the cached tracon and mark this direction as complete. */
 				*(jet[idir]) = t->tracon;
 				is_tracon[idir] = true;
 				continue;
@@ -286,7 +267,7 @@ build_disjunct(Sentence sent, Clause * cl, const char * string,
 		if (sat_solver || (!IS_GENERATION(sent->dict) || (' ' != string[0])))
 		{
 			ndis->word_string = string;
-			ndis->cost = cl->cost;
+			ndis->cost = cl->totcost;
 			ndis->is_category = 0;
 		}
 		else
@@ -300,8 +281,8 @@ build_disjunct(Sentence sent, Clause * cl, const char * string,
 			assert(sat_solver || ((ndis->category[0].num > 0) &&
 			       (ndis->category[0].num < 64*1024)),
 			       "Insane category %u", ndis->category[0].num);
-			ndis->category[0].cost = cl->cost;
-			// ndis->cost = cl->cost; No! clobbers memory!
+			ndis->category[0].cost = cl->totcost;
+			// ndis->cost = cl->totcost; No! clobbers memory!
 		}
 
 		ndis->originating_gword = (gword_set*)gs; /* XXX remove constness */
@@ -315,11 +296,9 @@ Disjunct *build_disjuncts_for_exp(Sentence sent, Exp* exp, const char *word,
                                   const gword_set *gs, float cost_cutoff,
                                   Parse_Options opts)
 {
-	Clause *c;
-	Disjunct * dis;
 	clause_context ct = { 0 };
-
 	ct.cost_cutoff = cost_cutoff;
+
 	if (unlikely(sent->Clause_pool == NULL))
 	{
 		ct.Clause_pool = pool_new(__func__, "Clause",
@@ -339,9 +318,9 @@ Disjunct *build_disjuncts_for_exp(Sentence sent, Exp* exp, const char *word,
 	}
 
 	// printf("%s\n", lg_exp_stringify(exp));
-	c = build_clause(exp, &ct, NULL);
+	Clause *c = build_clause(exp, &ct, NULL);
 	// print_clause_list(c);
-	dis = build_disjunct(sent, c, word, gs, cost_cutoff, opts);
+	Disjunct *dis = build_disjunct(sent, c, word, gs, cost_cutoff, opts);
 	// print_disjunct_list(dis);
 	pool_reuse(ct.Clause_pool);
 	pool_reuse(ct.Tconnector_pool);
@@ -366,7 +345,7 @@ GNUC_UNUSED static void print_clause_list(Clause * c)
 {
 	for (;c != NULL; c=c->next) {
 		printf("  Clause: ");
-		printf("(%4.2f, %4.2f) ", c->cost, c->maxcost);
+		printf("(%4.2f) ", c->totcost);
 		print_Tconnector_list(c->c);
 		printf("\n");
 	}
