@@ -11,10 +11,9 @@
 /*                                                                       */
 /*************************************************************************/
 
+#include <inttypes.h>                    // format macros
+
 #include "api-structures.h"              // for Sentence_s
-#ifdef DEBUG
-#include "build-disjuncts.h"             // allow using prt_exp_mem
-#endif
 #include "connectors.h"
 #include "dict-common/dict-structures.h" // expression_stringify
 #include "dict-common/dict-utils.h"      // size_of_expression
@@ -51,13 +50,14 @@
  */
 
 #define D_EXPRUNE 9
+#define D_PRINT_NUM_DISJUNCTS 5
 
 #ifdef DEBUG
 #define DBG(p, w, X) \
 	if (verbosity_level(+D_EXPRUNE))\
 	{\
 		err_msg(lg_Trace, "pass%d w%zu: ", p, w);\
-		err_msg(lg_Trace, X ": %s\n", lg_exp_stringify(x->exp));\
+		err_msg(lg_Trace, X ": %s\n", exp_stringify(x->exp));\
 	}
 #else /* !DEBUG */
 #define DBG(p, w, X)
@@ -172,14 +172,7 @@ static inline bool matches_S(connector_table **ct, int w, condesc_t * c)
 
 	for (e = ct[hash_S(c)]; e != NULL; e = e->next)
 	{
-		if (e->farthest_word <= 0)
-		{
-			if (w < -e->farthest_word) continue;
-		}
-		else
-		{
-			if (w > e->farthest_word) continue;
-		}
+		if (w > e->farthest_word) continue;
 		if (easy_match_desc(e->condesc, c)) return true;
 	}
 	return false;
@@ -204,8 +197,8 @@ static Exp* purge_Exp(exprune_context *ctxt, int, Exp *, char);
 static bool or_purge_operands(exprune_context *ctxt, int w, Exp *e, char dir)
 {
 #if NOTYET
-	const double nullexp_nonexistence = -9999;
-	double nullexp_mincost = nullexp_nonexistence;
+	const float nullexp_nonexistence = -9999;
+	float nullexp_mincost = nullexp_nonexistence;
 	int nullexp_count = 0;
 #endif
 
@@ -295,7 +288,7 @@ static Exp* purge_Exp(exprune_context *ctxt, int w, Exp *e, char dir)
 	{
 		if (e->dir == dir)
 		{
-			if (!matches_S(ctxt->ct, w, e->condesc))
+			if (!matches_S(ctxt->ct, (dir == '-') ? w : -w, e->condesc))
 			{
 				ctxt->N_deleted++;
 				return NULL;
@@ -309,7 +302,7 @@ static Exp* purge_Exp(exprune_context *ctxt, int w, Exp *e, char dir)
 	{
 		if (!and_purge_operands(ctxt, w, e, dir)) return NULL;
 	}
-	else /* if we are here, its OR_type */
+	else /* if we are here, it's OR_type */
 	{
 		if (!or_purge_operands(ctxt, w, e, dir)) return NULL;
 	}
@@ -337,7 +330,8 @@ static void zero_connector_table(exprune_context *ctxt)
  * This function puts connector c into the connector table
  * if one like it isn't already there.
  */
-static void insert_connector(exprune_context *ctxt, int farthest_word, condesc_t * c)
+static void insert_connector(exprune_context *ctxt, int farthest_word,
+                             condesc_t *c)
 {
 	unsigned int h;
 	connector_table *e;
@@ -371,11 +365,7 @@ static void insert_connectors(exprune_context *ctxt, int w, Exp * e, int dir)
 		if (e->dir == dir)
 		{
 			assert(NULL != e->condesc, "NULL connector");
-			Connector c = { .desc = e->condesc };
-
-			set_connector_length_limit(&c, ctxt->opts);
-			int farthest_word = (dir == '-') ? -MAX(0, w-c.length_limit) :
-				                              w+c.length_limit;
+			int farthest_word = (dir == '-') ? -e->farthest_word : e->farthest_word;
 			insert_connector(ctxt, farthest_word, e->condesc);
 		}
 	}
@@ -406,6 +396,23 @@ static char *print_expression_sizes(Sentence sent)
 	return dyn_str_take(e);
 }
 
+
+static void print_expression_disjunct_count(Sentence sent)
+{
+	uint64_t dcnt, t = 0;
+
+	for (WordIdx i = 0; i < sent->length; i++)
+	{
+		dcnt = 0;
+		for (const X_node *x = sent->word[i].x; x != NULL; x = x->next)
+			dcnt += count_clause(x->exp);
+		prt_error("%s(%"PRIu64") ", sent->word[i].alternatives[0], dcnt);
+		t += dcnt;
+	}
+	prt_error("\n\\");
+	prt_error("Total: %"PRIu64" disjuncts\n\n", t);
+}
+
 void expression_prune(Sentence sent, Parse_Options opts)
 {
 	size_t w;
@@ -421,6 +428,12 @@ void expression_prune(Sentence sent, Parse_Options opts)
 
 	DBG_EXPSIZES("Initial expression sizes\n%s", e);
 
+	if (verbosity_level(D_PRINT_NUM_DISJUNCTS))
+	{
+		prt_error("Debug: Before expression_prune():\n\\");
+		print_expression_disjunct_count(sent);
+	}
+
 	for (int pass = 0; ; pass++)
 	{
 		/* Left-to-right pass */
@@ -498,6 +511,12 @@ void expression_prune(Sentence sent, Parse_Options opts)
 	}
 
 	free_connector_table(&ctxt);
+
+	if (verbosity_level(D_PRINT_NUM_DISJUNCTS))
+	{
+		prt_error("Debug: After expression_prune():\n\\");
+		print_expression_disjunct_count(sent);
+	}
 }
 
 #if 0 // VERY_DEAD_NO_GOOD_IDEA
