@@ -1793,7 +1793,7 @@ Exp* SATEncoderConjunctionFreeSentences::PositionConnector2exp(const PositionCon
 bool SATEncoderConjunctionFreeSentences::sat_extract_links(Linkage lkg)
 {
   Disjunct *d;
-  int current_link = 0;
+  uint32_t current_link = 0;
 
   Exp **exp_word = (Exp **)alloca(_sent->length * sizeof(Exp *));
   memset(exp_word, 0, _sent->length * sizeof(Exp *));
@@ -1810,7 +1810,7 @@ bool SATEncoderConjunctionFreeSentences::sat_extract_links(Linkage lkg)
     if (_solver->model[_variables->linked(var->left_word, var->right_word)] != l_True)
       continue;
 
-    check_link_size(lkg);
+    assert(current_link < lkg->lasz, "Linkage array too small!");
     Link& clink = lkg->link_array[current_link];
     current_link++;
     clink.lw = var->left_word;
